/* ----------------------------------------------------------------------- *
 *   
 *   Copyright 2000-2008 H. Peter Anvin - All Rights Reserved
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as
 *   published by the Free Software Foundation, Inc.,
 *   59 Temple Place Ste 330, Boston MA 02111-1307, USA, version 2.1,
 *   incorporated herein by reference.
 *
 * ----------------------------------------------------------------------- */

/*
 * allocsize.c
 *
 * Detemine the "true" size of the specified allocation
 */

#include <assert.h>
#include <stdlib.h>
#include <inttypes.h>
#include <limits.h>
#include <string.h>

#include "lpsm.h"
#include "internals.h"
#include "bitops.h"

static size_t
lpsm_alloc_size_buddy(const void *ptr)
{
  struct buddy_params bp;
  bp = lpsm_find_alloc_buddy(ptr);

  return ((size_t)1) << (AH->arena_size_lg2-bp.rorder);
}

static size_t
lpsm_alloc_size_slab(const void *ptr)
{
  const struct slab_header *sh;
  const struct slab_info   *si;

  sh = (struct slab_header *)((uintptr_t)ptr & ~(BUDDY_SIZE_MIN-1));
  si = &AH->slab[sh->index];

  return si->size;
}

size_t
lpsm_alloc_size(const void *ptr)
{
  if ( !ptr )
    return 0;
  
  /* Buddy allocations are ALWAYS page-aligned, SLAB allocations never */
  if ( (uintptr_t)ptr & (BUDDY_SIZE_MIN-1) )
    return lpsm_alloc_size_slab(ptr);
  else
    return lpsm_alloc_size_buddy(ptr);
}
